/**
 * Created by Mihail on 5/30/18.
 */
'use strict';

const fetch = require('node-fetch');
const FormData = require('form-data');

const qs = require('qs');

class Hasoffers {
    constructor ({networkId = "", networkToken = "", fetcher = fetch, config = {}  } = {}) {
        this.networkId  = networkId;
        this.networkToken  = networkToken;
        this.config = Object.assign({}, {
            method: 'GET',
            headers: {
                Accept: 'application/json',
                'Accept-Charset': 'utf-8'
            }
        }, config);

        this.fetcher = fetcher;
        this.url = `https://${this.networkId}.api.hasoffers.com`;
    }
    getOffer (id) {
        return createRequest({
            fetcher: this.fetcher,
            url: `${this.url}/Apiv3/json?NetworkToken=${this.networkToken}&Target=Offer&Method=findById&id=${id}`,
            config: this.config
        })
    }
    getAllCountries () {
        return createRequest({
            fetcher: this.fetcher,
            url: `${this.url}/Apiv3/json?NetworkToken=${this.networkToken}&Target=Application&Method=findAllCountries`,
            config: this.config
        })
    }
    createOffer (data) {
        let queryString = Object.keys(data).map(key => `data[${key}]` + '=' + encodeURIComponent(data[key])).join('&');
        return createRequest({
            fetcher: this.fetcher,
            url: `${this.url}/Apiv3/json?NetworkToken=${this.networkToken}&Target=Offer&Method=create&${queryString}`,
            config: this.config
        })
    }
    updateOfferField (id, field, value) {
        let queryValue = encodeURIComponent(value);
        return createRequest({
            fetcher: this.fetcher,
            url: `${this.url}/Apiv3/json?NetworkToken=${this.networkToken}&Target=Offer&Method=updateField&id=${id}&field=${field}&value=${queryValue}`,
            config: this.config
        })
    }
    updateOffer (id, data) {
        let queryString = Object.keys(data).map(key => `data[${key}]` + '=' + encodeURIComponent(data[key])).join('&');
        return createRequest({
            fetcher: this.fetcher,
            url: `${this.url}/Apiv3/json?NetworkToken=${this.networkToken}&Target=Offer&Method=update&id=${id}&${queryString}`,
            config: this.config
        })
    }
    createOfferTag (offer_id, tag_id) {
        return createRequest({
            fetcher: this.fetcher,
            url: `${this.url}/Apiv3/json?NetworkToken=${this.networkToken}&Target=Tag&Method=addToOffer&&offer_id=${offer_id}&tag_id=${tag_id}`,
            config: this.config
        })
    }
    createGeoCountries (id, data) {
        let queryString = Object.keys(data).map(key => `location_ids[Countries][]` + '=' + encodeURIComponent(data[key])).join('&');

        return createRequest({
            fetcher: this.fetcher,
            url: `${this.url}/Apiv3/json?NetworkToken=${this.networkToken}&Target=Offer&Method=setGeoTargeting&id=${id}&${queryString}`,
            config: this.config
        })
    }
    createGeoCities (id, data) {
        let queryString = Object.keys(data).map(key => `location_ids[Cities][]` + '=' + encodeURIComponent(data[key])).join('&');

        return createRequest({
            fetcher: this.fetcher,
            url: `${this.url}/Apiv3/json?NetworkToken=${this.networkToken}&Target=Offer&Method=setGeoTargeting&id=${id}&${queryString}`,
            config: this.config
        })
    }
    createOfferFile (offer_id, filename, display, type, file) {
        let new_data = {
            type: encodeURIComponent(type),
            display: encodeURIComponent(display),
            filename: encodeURIComponent(filename),
            offer_id: encodeURIComponent(offer_id),
        };
        const form = new FormData();
        return createRequest({
            fetcher: this.fetcher,
            url: `${this.url}/Apiv3/json?NetworkToken=${this.networkToken}&Target=OfferFile&Method=create&data[type]=${new_data.type}&data[display]=${new_data.display}&data[file][name]=${new_data.filename}&data[offer_id]=${new_data.offer_id}&return_object=1`,
            config: {
                method: 'POST',
                body: form.append(filename, file),
                headers: form.getHeaders()
            }
        })
    }
}
const createRequest = (args = {}) => {
    const { url, config, query, fetcher } = args;
    return fetch(`${url}${query ? `?${qs.stringify(query)}` : ''}`, config).then(res =>
        res.json()
    )
};

module.exports = Hasoffers;